@@ -23,6 +23,7 @@ import serial
 
 
 
+
 def packVarSize(value):
     bytes = bytearray([])
     tmp =0
@@ -99,6 +100,58 @@ class PolySerial (threading.Thread):
                     data = self.serialPort.read()
                     self.iface.feedEncodedBytes(data)
 
+class PolyTcp (threading.Thread):
+    def __init__(self, iface, localPort ):
+        threading.Thread.__init__(self)
+        self.iface = iface
+        self.localPort = localPort
+        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+       # self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
+        self.socket.setblocking(0)
+        self.socket.bind(("127.0.0.1", self.localPort))
+        self.iface.print(self.iface.name + " Listening on port: " + str(self.socket.getsockname()[1]))
+        self.host = 0
+        self.server = server
+
+    def __del__(self):
+        self.socket.close()
+        self.join()
+
+    def close(self):
+        self.socket.close()
+
+    def connect(self, hostIp, hostPort):
+        self.iface.print(self.iface.name + " Connecting to " + hostIp + ":"+str(hostPort))
+        self.host = (hostIp, hostPort)
+        try:
+            self.socket.connect(self.host)
+        except Exception  as e:
+            self.iface.print(str(e))
+
+    def send(self, data):
+        if not self.host == 0:
+            try:
+                #self.iface.print(str(self.host))
+                #self.iface.print(" >>> " + ''.join(' {:02x}'.format(x) for x in data))
+                self.socket.send(data)
+            except Exception  as e:
+                self.iface.print(str(e))
+
+    def run(self):
+
+        self.socket.listen()
+        while True:
+            try:
+                data, address = self.socket.recv(1024)
+                if data:
+                    if self.host == 0:
+                        self.host = address
+                        self.iface.service.print(" Connection Accepted: " + str(self.host))
+                    #self.iface.print(" <<< " + ''.join(' {:02x}'.format(x) for x in data))
+                    self.iface.feedEncodedBytes(data)
+            except IOError as e:  # and here it is handeled
+                if e.errno == errno.EWOULDBLOCK:
+                    pass
 
 class PolyUdp (threading.Thread):
     def __init__(self, iface, localPort ):
@@ -108,13 +161,13 @@ class PolyUdp (threading.Thread):
         self.socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
         self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
         self.socket.setblocking(0)
-        self.socket.bind(("127.0.0.1", self.localPort))
-        self.iface.print(self.iface.name + " Listening on port: " + str(localPort))
+        self.socket.bind(("0.0.0.0", self.localPort))
+        self.iface.print(self.iface.name + " Listening on port: " + str(self.socket.getsockname()[1]))
         self.host = 0
 
     def __del__(self):
         self.socket.close()
-        threading.Thread.__del__(self)
+        self.join()
 
     def close(self):
         self.socket.close()
@@ -125,8 +178,12 @@ class PolyUdp (threading.Thread):
 
     def send(self, data):
         if not self.host == 0:
-            #self.iface.print(" >>> " + ''.join(' {:02x}'.format(x) for x in data))
-            self.socket.sendto(data, self.host)
+            try:
+                #self.iface.print(str(self.host))
+                #self.iface.print(" >>> " + ''.join(' {:02x}'.format(x) for x in data))
+                self.socket.sendto(data, self.host)
+            except Exception  as e:
+                self.iface.print(str(e))
 
     def run(self):
         while True:
@@ -171,6 +228,15 @@ class PolyField:
                 if self.desc.isString:
                     self.len = len(val)
                     self.values[0] = val
+
+                elif self.desc.isMask and '|' in val:
+                    self.len = 1
+                    self.values[0] = 0
+                    arrVals = val.split('|')
+                    for v in arrVals:
+                        v = v.strip()
+                        if v in self.desc.valDict:
+                            self.values[0] = self.values[0] | self.desc.valDict[v]
                 else:
                     self.len = 1
                     self.values[0] = int(val, 0)
@@ -395,22 +461,81 @@ class PolyIface:
         self.name = ""
         self.lastToken = 0
 
+
         if not connStr == "":
-            words = connStr.split(':')
+            words = connStr.split(':')  
+            localPort = 0    #randomly picks a free port
+            remotePort = -1
+            remoteHost = '127.0.0.1'
+
+            #udp:local_port:remote_port
+            #udp:local_port:remote_ip:remote_port
+            #udp:remote_ip:remote_port
 
             if words[0] == 'udp':
                 try:
-                    self.coms = PolyUdp(self, int(words[1]))
-                    self.name = "UDP"
-                    if len(words) == 3:
-                        self.coms.connect('127.0.0.1', int(words[2]))
-                    if len(words) == 4:
-                        self.coms.connect(words[2], int(words[3]))
-                    
-                    self.coms.daemon = True
-                    self.coms.start()
+                    if(words[1].isnumeric()): 
+                        localPort = int(words[1])
+                        if len(words) == 3:              #udp:local_port:remote_port
+                            localPort = int(words[1])
+                            remotePort = int(words[2])
+                        if len(words) == 4:              #udp:local_port:remote_ip:remote_port
+                            remoteHost = words[2]
+                            remotePort = int(words[3])
+                    else:                                #udp:remote_ip:remote_port
+                        remoteHost = words[1]
+                        remotePort = int(words[2])
+
                 except:
-                    self.print( "Invalid connection string\n udp:local-port to open a port\n udp:local-port:ip:remote-port to target remote port ")
+                    udp_help = ""
+                    udp_help+= "Invalid connection string. Options:\n"
+                    udp_help+= "\t[udp:local-port] to listen on a port\n"
+                    udp_help+= "\t[udp:host:remote-port] to target remote port on a host and use the default local port\n"
+                    udp_help+= "\t[udp:local-port:host:remote-port] to target remote port on host and specify local hose\n"
+                    udp_help+= "\t[udp:remote-port] to target port on local host\n"
+                    udp_help+= "\t[udp:local-port:remote-port] to target port on local host, specifying local port\n"
+                    self.print( udp_help)
+
+                self.coms = PolyUdp(self, localPort)
+                self.name = "UDP"
+                if remotePort > -1:
+                    self.coms.connect(remoteHost, remotePort)
+                
+                self.coms.daemon = True
+                self.coms.start()
+            
+            if words[0] == 'tcp':
+                self.print( "***WARNING*** TCP mode under construction")
+                try:
+                    if(words[1].isnumeric()): 
+                        localPort = int(words[1])
+                        if len(words) == 3:              #udp:local_port:remote_port
+                            localPort = int(words[1])
+                            remotePort = int(words[2])
+                        if len(words) == 4:              #udp:local_port:remote_ip:remote_port
+                            remoteHost = words[2]
+                            remotePort = int(words[3])
+                    else:                                #udp:remote_ip:remote_port
+                        remoteHost = words[1]
+                        remotePort = int(words[2])
+
+                except:
+                    tcp_help = ""
+                    tcp_help+= "Invalid connection string. Options:\n"
+                    tcp_help+= "\t[tcp:local-port] to listen on a port\n"
+                    tcp_help+= "\t[tcp:host:remote-port] to target remote port on a host and use the default local port\n"
+                    tcp_help+= "\t[tcp:local-port:host:remote-port] to target remote port on host and specify local hose\n"
+                    tcp_help+= "\t[tcp:remote-port] to target port on local host\n"
+                    tcp_help+= "\t[tcp:local-port:remote-port] to target port on local host, specifying local port\n"
+                    self.print( tcp_help)
+
+                self.coms = PolyTcp(self, localPort)
+                self.name = "TCP"
+                if remotePort > -1:
+                    self.coms.connect(remoteHost, remotePort)
+                
+                self.coms.daemon = True
+                self.coms.start()
 
             elif words[0] == 'serial':
                 try:
